/*
 * gexiv2-log-private.h
 *
 * Author(s)
 * Jim Nelson <jim@yorba.org>
 *
 * This is free software. See COPYING for details.
 */

#ifndef __GEXIV2_LOG_PRIVATE_H__
#define __GEXIV2_LOG_PRIVATE_H__

#include <gexiv2/gexiv2-log.h>
#include <exiv2/error.hpp>

G_BEGIN_DECLS

gboolean        gexiv2_log_is_handler_installed(void);

G_END_DECLS

#endif /* __GEXIV2_LOG_PRIVATE_H__ */
